import re


class CNPJ:

    def __init__(self, numbers):
        self.numbers = numbers

    def unformatted(self):
        return re.sub(r"\D", "", self.numbers)

    def matchPattern(self, pattern=r"^\d{2}.\d{3}.\d{3}\/\d{4}-\d{2}$"):
        return re.match(pattern, self.numbers)

    def isValid(self, pattern=r"^\d{2}.\d{3}.\d{3}\/\d{4}-\d{2}$"):

        if not self.matchPattern(pattern): return False

        cnpj = self.unformatted()
        if len(cnpj) != 14: return False

        numbers = map(int, cnpj)
        new = numbers[:12]
        prod = [5, 4, 3, 2, 9, 8, 7, 6, 5, 4, 3, 2]
        while len(new) < 14:
            r = sum([x * y for (x, y) in zip(new, prod)]) % 11
            if r > 1:
                f = 11 - r
            else:
                f = 0
            new.append(f)
            prod.insert(0, 6)

        return bool(new == numbers)