from ..utils.enum import Enum


class Environment(Enum):

    production = "production"
    development = "development"

    @classmethod
    def current(cls):
        from ..utils.metadata import Metadata
        projectId = Metadata.getProjectId()
        return cls.development if "dev" in projectId else cls.production


class Microservice(Enum):

    transfer = "transfer"
    charge = "charge"
    refund = "refund"
    workspace = "workspace"
    bank = "bank"
    gateway = "gateway"
    edi = "edi"

    @classmethod
    def getBaseUrl(cls, ms, environment=Environment.current()):
        return "https://%s.appspot.com" % cls.getProjectId(ms, environment=environment)

    @classmethod
    def getProjectId(cls, ms, environment=Environment.current()):
        return {
            Environment.production: _productionProjectId.get(ms),
            Environment.development: _developmentProjectId.get(ms)
        }[environment]


class Webapp:

    @classmethod
    def getBaseUrl(cls, environment=Environment.current()):
        return {
            Environment.production:  "https://darwin-web.appspot.com",
            Environment.development: "https://darwin-web-dev.appspot.com"
        }[environment]


_developmentProjectId = {
    Microservice.transfer:  "api-ms-transfer-dev",
    Microservice.charge:    "api-ms-charge-dev",
    Microservice.refund:    "api-ms-refund-dev",
    Microservice.workspace: "api-ms-workspace-dev",
    Microservice.bank:      "api-ms-bank-dev",
    Microservice.gateway:   "api-ms-gateway-dev",
    Microservice.edi:       "api-ms-edi-dev"
}

_productionProjectId = {
    Microservice.transfer:  "api-ms-transfer",
    Microservice.charge:    "api-ms-charge",
    Microservice.refund:    "api-ms-refund",
    Microservice.workspace: "api-ms-workspace",
    Microservice.bank:      "api-ms-bank",
    Microservice.gateway:   "api-ms-gateway",
    Microservice.edi:       "api-ms-edi"
}
