from .enum import Enum


class RefundPermission(Enum):

    viewTeams = "refund/viewTeams"
    createTeams = "refund/createTeams"
    deleteTeams = "refund/deleteTeams"
    editTeams = "refund/editTeams"
    payRefunds = "refund/payRefunds"
    viewRefunds = "refund/viewRefunds"
    createRefunds = "refund/createRefunds"
    deleteRefunds = "refund/deleteRefunds"
    approveRefunds = "refund/approveRefunds"
    viewRoles = "refund/viewRoles"
    editRoles = "refund/editRoles"


class BankPermission(Enum):

    viewStatement = "bank/viewStatement"
    viewBalance = "bank/viewBalance"
    viewTransaction = "bank/viewTransaction"
    createTransaction = "bank/createTransaction"


class TransferPermission(Enum):

    viewRequests = "transfer/viewRequest"
    createRequest = "transfer/createRequest"


class ChargePermission(Enum):

    viewSlips = "charge/viewSlips"
    createSlip = "charge/createSlip"


class WorkspacePermission(Enum):

    viewRoles = "workspace/viewRoles"
    editRoles = "workspace/editRoles"


class WorkspaceRole(Enum):

    admin = "admin"
    financial = "financial"
    member = "member"

    @classmethod
    def permissions(self, role):
        return {
            WorkspaceRole.member: [
                    RefundPermission.viewTeams,
                    RefundPermission.viewRoles,
                    WorkspacePermission.viewRoles
                ]
            ,
            WorkspaceRole.financial: [
                    RefundPermission.viewTeams,
                    RefundPermission.viewRoles,
                    WorkspacePermission.viewRoles
                ]
                + BankPermission.values()
                + TransferPermission.values()
                + ChargePermission.values()
            ,
            WorkspaceRole.admin:
                RefundPermission.values()
                + BankPermission.values()
                + TransferPermission.values()
                + ChargePermission.values()
                + WorkspacePermission.values()
            ,
        }.get(role, [])

    @classmethod
    def contains(cls, permission, inRoles):
        for role in inRoles:
            if permission in cls.permissions(role):
                return True
        return False