import os
from ..libs import cloudstorage as gcs
from google.appengine.api import app_identity


class Storage:

    @classmethod
    def defaulfBucket(cls):
        return os.environ.get("", app_identity.get_default_gcs_bucket_name())

    @classmethod
    def bucketPath(cls, fileName="", filePath=""):
        return "/" + cls.defaulfBucket() + filePath + "/" + fileName

    @classmethod
    def baseURL(cls):
        return "https://storage.googleapis.com"

    @classmethod
    def publicUrl(cls, bucketPath):
        return str(cls.baseURL() + bucketPath)

    @classmethod
    def save(cls, file, bucketPath, contentType=None, isPublic=True):
        options = {"x-goog-acl": "public-read"} if isPublic else None
        gcsFile=gcs.open(bucketPath, "w", content_type=contentType, options=options)
        gcsFile.write(file)
        gcsFile.close()
        return cls.publicUrl(bucketPath)

    @classmethod
    def read(cls, bucketPath):
        gcsFile = gcs.open(bucketPath)
        content = gcsFile.read()
        gcsFile.close()
        return content

    @classmethod
    def delete(cls, bucketPath):
        try:
            gcs.delete(bucketPath)
        except gcs.NotFoundError:
            pass

    @classmethod
    def exists(cls, bucketPath):
        try:
            gcs.stat(bucketPath)
            return True
        except:
            return False

    @classmethod
    def list(cls, path):
        for bucket in gcs.listbucket(path):
            yield bucket.filename

    @classmethod
    def move(cls, currentFilePath, newFolderPath):
        fileName = currentFilePath.split('/')[-1]
        newFilePath = newFolderPath + fileName
        gcs.copy2(
            src=currentFilePath,
            dst=newFilePath
        )
        gcs.delete(currentFilePath)
